local dvireader = require "dvireader"
local mkutils = require "mkutils"
local filter = require "make4ht-filter"

-- mapping between tex4ht image names and hashed image names
local output_map = {}
local dvisvgm_options = "-n --exact -c 1.15,1.15"
local function make_hashed_name(base, hash)
  return base .. "-" ..hash..".svg"
end


-- process output of dvisvgm and find output page numbers and corresponding files
local function get_generated_pages(output)
  local pages = {}
  local pos = 1
  local pos, finish, page = string.find(output, "processing page (%d+)", pos)
  while(pos) do
    pos, finish, file = string.find(output, "output written to ([^\n]+)", finish)
    pages[tonumber(page)] = file
    pos, finish, page = string.find(output, "processing page (%d+)", finish)
  end
  return pages
end

local function execute_dvisvgm(idvfile, pages)
  if #pages < 1 then return nil, "No pages to convert" end
  local pagesequence = table.concat(pages, ",")
  -- the stderr from dvisvgm must be redirected and postprocessed
  local dvisvgmlog = idvfile:gsub("idv$", "dlog")
  -- local dvisvgm = io.popen("dvisvgm -v4 -n --exact -c 1.15,1.15 -p " .. pagesequence .. " " .. idvfile, "r")
  local command = "dvisvgm -v4 " .. dvisvgm_options .. " -p " .. pagesequence .. " " .. idvfile .. " 2> " .. dvisvgmlog
  print(command)
  os.execute(command)
  local f = io.open(dvisvgmlog, "r")
  if not f then return nil, "Cannot read dvisvgm log" end
  local output = f:read("*all")
  f:close()
  -- find page numbers and corresponding files for generated images
  return get_generated_pages(output)
end

local function get_dvi_pages(arg)
  -- list of pages to convert in this run
  local to_convert = {}
  local idv_file = arg.input .. ".idv"
  dvisvgm_options = arg.options or dvisvgm_options
  local f = io.open(idv_file, "r")
  if not f then return nil, "Cannot open idv file: " .. idv_file end
  local content = f:read("*all")
  f:close()
  local dvi_pages = dvireader.get_pages(content)
  -- we must find page numbers and output name sfor the generated images
  local lg = mkutils.parse_lg(arg.input ..".lg")
  for _, name in ipairs(lg.images) do
    local page = tonumber(name.page)
    local hash = dvi_pages[page]
    local tex4ht_name = name.output
    local output_name = make_hashed_name(arg.input, hash)
    output_map[tex4ht_name] = output_name
    if not mkutils.file_exists(output_name) then
      print(output_name)
      to_convert[#to_convert+1] = page
    end
  end
  local generated_files, msg = execute_dvisvgm(idv_file, to_convert)
  if not generated_files then
    return nil, msg
  end

  -- rename the generated files to the hashed filenames
  for page, file in pairs(generated_files) do
    os.rename(file, make_hashed_name(arg.input, dvi_pages[page]))
  end

end

-- this must be used in the .mk4 file as
-- Make:dvisvgm_hashes {}
Make:add("dvisvgm_hashes", function(arg)
  get_dvi_pages(arg)
end, {})

-- replace original image names with hashed names
local executed = false
Make:match(".*", function(arg)
  if not executed then
    executed = true
    local lgfiles = Make.lgfile.files
    for i, filename in ipairs(lgfiles) do
      local replace = output_map[filename]
      if replace then
        lgfiles[i] = replace
      end
     end
  end
end)

-- fix src attributes
local process = filter {
  function(str)
    return str:gsub('src="([^"]+)', function(filename)
      local newname = output_map[filename] or filename
      print("newname", newname)
      return 'src="'.. newname 
    end)
  end
}
  
Make:match("html$", process)

-- disable the image processing
Make:image(".", function() return "" end)
